@@ -63,6 +63,9 @@ getOrientation <- function(xyz, delta=0.0001) {
 swapDimension <- function(img, dcm) {
   imagePositionPatient <-
     header2matrix(extractHeader(dcm$hdr, "ImagePositionPatient", FALSE), 3)
+  if (nrow(imagePositionPatient) != nsli(img)) {
+    imagePositionPatient <- attributes(img)$ipp
+  }
   imageOrientationPatient <-
     header2matrix(extractHeader(dcm$hdr, "ImageOrientationPatient", FALSE), 6)
   ## Ensure all rows of imageOrientationPatient are identical!
@@ -80,81 +83,98 @@ swapDimension <- function(img, dcm) {
   Y <- ncol(img)
   Z <- dim(img)[3]
   W <- dim(img)[4]
-  ld <- length(dim(img))
+  ld <- as.numeric(length(dim(img)))
+  ## AXIAL
   if (is.axial(imageOrientationPatient)) {
-    if (first.row %in% c("A","P")) {
-      index <- c(2,1,3)
+    if (unlist(strsplit(first.row,""))[1] %in% c("A","P")) {
+      if (identical(ld, 3)) {
+        index <- c(2,1,3)
+      }
+      if (identical(ld, 4)) {
+        index <- c(2,1,3,4)
+      }
       img <- aperm(img, index)
-      pixdim <- pixdim[index]
+      pixdim <- pixdim[index[1:3]]
     }
-    if (first.row == "R") {
-      img <- switch(as.character(ld), "3" = img[X:1,,], "4" = img[X:1,,,])
+    if (identical(unlist(strsplit(first.row,""))[1], "R")) {
+      img <- switch(as.character(ld), "3" = img[X:1,,], "4" = img[X:1,,,],
+                    stop("Dimension \"DIM\" incorrectly specified."))
     }
-    if (first.col == "A") {
+    if (identical(unlist(strsplit(first.col,""))[1], "A")) {
       img <- switch(as.character(ld), "3" = img[,Y:1,], "4" = img[,Y:1,,])
     }
     ## The z-axis is increasing toward the HEAD of the patient.
     z.index <- order(imagePositionPatient[,3])
-    ## x <- do.call("[<-", c(list(x), dimnames(y), list(y)))
-    img <- switch(as.character(ld), "3" = img[,,z.index], "4" = img[,,z.index,])
-    imagePositionPatient <<- imagePositionPatient[z.index,]
+    img <- switch(as.character(ld), "3" = img[,,z.index], "4" = img[,,Z:1,])
   }
+  ## CORONAL
   if (is.coronal(imageOrientationPatient)) {
-    if (first.row %in% c("H","F")) {
-      index <- c(2,1,3)
+    if (unlist(strsplit(first.row,""))[1] %in% c("H","F")) {
+      if (identical(ld, 3)) {
+        index <- c(2,1,3)
+      }
+      if (identical(ld, 4)) {
+        index <- c(2,1,3,4)
+      }
       img <- aperm(img, index)
-      pixdim <- pixdim[index]
+      pixdim <- pixdim[index[1:3]]
     }
-    if (first.row == "R") {
-      img <- switch(as.character(length(dim(img))),
-                  "3" = img[X:1,,],
-                  "4" = img[X:1,,,],
-                  stop("Dimension parameter \"DIM\" incorrectly specified."))
+    if (identical(unlist(strsplit(first.row,""))[1], "R")) {
+      img <- switch(as.character(ld), "3" = img[X:1,,], "4" = img[X:1,,,],
+                    stop("Dimension \"DIM\" incorrectly specified."))
     }
-    if (first.col == "H") {
-      img <- img[,Y:1,]
+    if (identical(unlist(strsplit(first.col,""))[1], "H")) {
+      img <- switch(as.character(ld), "3" = img[,Y:1,], "4" = img[,Y:1,,])
     }
     ## The y-axis is increasing to the posterior side of the patient.
     z.index <- order(imagePositionPatient[,2])
-    ## img <- img[,,z.index]
-    img <- switch(as.character(length(dim(img))),
-                  "3" = img[,,z.index],
-                  "4" = img[,,z.index,],
-                  stop("Dimension parameter \"DIM\" incorrectly specified."))
-    imagePositionPatient <<- imagePositionPatient[z.index,]
-    ##
-    index <- c(1,3,2)
+    img <- switch(as.character(ld), "3" = img[,,z.index], "4" = img[,,Z:1,])
+    if (identical(ld, 3)) {
+      index <- c(1,3,2)
+    }
+    if (identical(ld, 4)) {
+      index <- c(1,3,2,4)
+    }
     img <- aperm(img, index) # re-organize orthogonal views
-    pixdim <- pixdim[index]
+    pixdim <- pixdim[index[1:3]]
   }
+  ## SAGITTAL
   if (is.sagittal(imageOrientationPatient)) {
-    if (first.row %in% c("H","F")) {
-      index <- c(2,1,3)
+    if (unlist(strsplit(first.row,""))[1] %in% c("H","F")) {
+      if (identical(ld, 3)) {
+        index <- c(2,1,3)
+      }
+      if (identical(ld, 4)) {
+        index <- c(2,1,3,4)
+      }
       img <- aperm(img, index)
-      pixdim <- pixdim[index]
+      pixdim <- pixdim[index[1:3]]
     }
-    if (first.row == "P") {
-      img <- img[X:1,,]
+    if (identical(unlist(strsplit(first.row,""))[1], "P")) {
+      img <- switch(as.character(ld), "3" = img[X:1,,], "4" = img[X:1,,,],
+                    stop("Dimension \"DIM\" incorrectly specified."))
     }
-    if (first.col == "H") {
-      img <- img[,Y:1,]
+    if (identical(unlist(strsplit(first.col,""))[1], "H")) {
+      img <- switch(as.character(ld), "3" = img[,Y:1,], "4" = img[,Y:1,,])
     }
     ## The x-axis is increasing to the left hand side of the patient.
     z.index <- order(imagePositionPatient[,1])
-    ## img <- img[,,z.index]
-    img <- switch(as.character(length(dim(img))),
-                  "3" = img[,,z.index],
-                  "4" = img[,,z.index,],
+    img <- switch(as.character(ld), "3" = img[,,z.index], "4" = img[,,Z:1,],
                   stop("Dimension parameter \"DIM\" incorrectly specified."))
-    imagePositionPatient <<- imagePositionPatient[z.index,]
-    ## 
-    index <- c(3,1,2)
+    if (identical(ld, 3)) {
+      index <- c(3,1,2)
+    }
+    if (identical(ld, 4)) {
+      index <- c(3,1,2,4)
+    }
     img <- aperm(img, index) # re-organize orthogonal views
-    pixdim <- pixdim[index]
+    pixdim <- pixdim[index[1:3]]
   }
+  imagePositionPatient <<- imagePositionPatient[z.index,]
   if (any(is.na(imagePositionPatient))) {
     stop("Missing values are present in ImagePositionPatient.")
   }
+  attr(img,"ipp") <- imagePositionPatient
   attr(img,"pixdim") <- pixdim
   return(img)
 }
